/**
 * @file datamux_ctrl.c
 *
 * @brief @{Source file containing the data MUX control functions for the
 * CCG firmware application.
 *
 * The current implementation is for the embedded USB 3.1/DP 1.2 MUX in the
 * CCG4M device.@}
 */

/*
 * Copyright (2014-2017), Cypress Semiconductor Corporation or a subsidiary of
 * Cypress Semiconductor Corporation. All rights reserved.
 *
 * This software, including source code, documentation and related materials
 * (“Software”), is owned by Cypress Semiconductor Corporation or one of its
 * subsidiaries (“Cypress”) and is protected by and subject to worldwide patent
 * protection (United States and foreign), United States copyright laws and
 * international treaty provisions. Therefore, you may use this Software only
 * as provided in the license agreement accompanying the software package from
 * which you obtained this Software (“EULA”).
 *
 * If no EULA applies, Cypress hereby grants you a personal, nonexclusive,
 * non-transferable license to copy, modify, and compile the Software source
 * code solely for use in connection with Cypress’s integrated circuit
 * products. Any reproduction, modification, translation, compilation, or
 * representation of this Software except as specified above is prohibited
 * without the express written permission of Cypress. Disclaimer: THIS SOFTWARE
 * IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress reserves the
 * right to make changes to the Software without notice. Cypress does not
 * assume any liability arising out of the application or use of the Software
 * or any product or circuit described in the Software. Cypress does not
 * authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death (“High Risk Product”). By
 * including Cypress’s product in a High Risk Product, the manufacturer of such
 * system or application assumes all risk of such use and in doing so agrees to
 * indemnify Cypress against all liability.
 */

#include <pd.h>
#include <alt_modes_mngr.h>
#include <app.h>
#include <timer.h>

/* Definitions for internal data MUX in CCG4M. */
#define PI3USB_VENDOR_ID                    (0x00)
#define PI3USB_MUX_ISOLATE                  (0x0)
#define PI3USB_DP_4_LANE_CFG                (0x1)
#define PI3USB_SS_USB_ONLY_CFG              (0x2)
#define PI3USB_DP_2_LANE_CFG                (0x3)

/* Polarity state offset in the current mux state. */
#define MUX_STATE_POLARITY_OFFSET               (7u)

/* Macro to create MUX_STATE value from polarity and current MUX configuration. */
#define MAKE_MUX_STATE(polarity, cfg)       (((polarity) << MUX_STATE_POLARITY_OFFSET) | (cfg))

/* I2C slave address assigned to the data MUX for each port. */
static const uint8_t mux_i2c_address[NO_OF_TYPEC_PORTS] =
{
    0x54    /* This is the address for the internal MUX used for Port 0 in the CCG4M device. */
};

/* Current mux state. */
static uint8_t mux_state[NO_OF_TYPEC_PORTS] =
{
    MUX_CONFIG_ISOLATE
};

/* MUX access timeout indication. */
static volatile bool mux_xfer_timeout = false;

/* Timer callback used for I2C transactions to the MUX. */
static void mux_xfer_timer_cb(uint8_t port, timer_id_t id)
{
    (void)port;
    (void)id;

    /*
     * I2C transmission to MUX continues longer than timeout. Slave doesn't
     * respond.
     */
    mux_xfer_timeout = true;
}

static bool Mux_I2C_Write(uint8_t addr, uint8_t *buffer, uint32_t count)
{
    uint32_t i;
    uint8_t  status = false;

    /* Clear the timeout flag and start a timer. */
    mux_xfer_timeout = false;
    timer_start (0, MUX_I2C_TIMER, MUX_I2C_TIMER_PERIOD, mux_xfer_timer_cb);

    /* If bus is free, generate Start condition */
    if ((MUX_CTRL_I2C_STATUS_REG & MUX_CTRL_I2C_STATUS_BUS_BUSY) == 0)
    {
        /* Assume operation passed for now. */
        status = true;

        /* TX and RX FIFO have to be EMPTY */
        MUX_CTRL_TX_FIFO_WR_REG = (uint32_t)(addr << 1); /* Put address in TX FIFO */
        MUX_CTRL_ClearMasterInterruptSource (MUX_CTRL_INTR_MASTER_ALL);
        MUX_CTRL_I2C_MASTER_GENERATE_START;

        /* Wait for an ACK from the MUX. */
        while (!MUX_CTRL_CHECK_INTR_MASTER(MUX_CTRL_INTR_MASTER_I2C_ACK |
                                           MUX_CTRL_INTR_MASTER_I2C_NACK |
                                           MUX_CTRL_INTR_MASTER_I2C_ARB_LOST |
                                           MUX_CTRL_INTR_MASTER_I2C_BUS_ERROR) && (!mux_xfer_timeout));

        /* Transfer the remaining data out to the MUX. */
        for (i = 0; ((!mux_xfer_timeout) && (i < count)); i++)
        {
            MUX_CTRL_TX_FIFO_WR_REG = buffer[i];
            while (!MUX_CTRL_CHECK_INTR_MASTER(MUX_CTRL_INTR_MASTER_I2C_ACK |
                                               MUX_CTRL_INTR_MASTER_I2C_NACK |
                                               MUX_CTRL_INTR_MASTER_I2C_ARB_LOST |
                                               MUX_CTRL_INTR_MASTER_I2C_BUS_ERROR) && (!mux_xfer_timeout));
        }
    }

    /* Send a STOP to the slave. */
    MUX_CTRL_I2C_MASTER_GENERATE_STOP;
    while (!MUX_CTRL_CHECK_INTR_MASTER(MUX_CTRL_INTR_MASTER_I2C_STOP |
                                       MUX_CTRL_INTR_MASTER_I2C_ARB_LOST |
                                       MUX_CTRL_INTR_MASTER_I2C_BUS_ERROR) &&
           (!mux_xfer_timeout));

    /* Check the results of the address phase. */
    if (MUX_CTRL_CHECK_INTR_MASTER(MUX_CTRL_INTR_MASTER_I2C_NACK |
                                   MUX_CTRL_INTR_MASTER_I2C_ARB_LOST |
                                   MUX_CTRL_INTR_MASTER_I2C_BUS_ERROR) ||
        (mux_xfer_timeout))
    {
        /* Transaction failed. Reset the SCB block and return error. */
        MUX_CTRL_CTRL_REG &= ((uint32) ~MUX_CTRL_CTRL_ENABLED);
        MUX_CTRL_CTRL_REG |= ((uint32)  MUX_CTRL_CTRL_ENABLED);
        status = false;
    }

    timer_stop(0, MUX_I2C_TIMER);
    return (status);
}

/* Update the data mux settings as required. */
bool mux_ctrl_set_cfg(uint8_t port, mux_select_t cfg, uint8_t polarity)
{
    uint8_t tmp_buf[2u];
    bool    status = false;

    /*
     * Compare  new and recent mux states. Write to the MUX only if the config
     * is changing, or if the desired configuration is ISOLATE (to handle the
     * start-up case).
     */
    if ((cfg == MUX_CONFIG_ISOLATE) || (MAKE_MUX_STATE(polarity, cfg) != mux_state[port]))
    {
        tmp_buf[0] = PI3USB_VENDOR_ID;
        tmp_buf[1] = polarity;

        switch(cfg)
        {
            case MUX_CONFIG_ISOLATE:
                /* Set mux to powerdown state. */
                tmp_buf[1] |= (PI3USB_MUX_ISOLATE << 1);
                status = Mux_I2C_Write (mux_i2c_address[port], tmp_buf, sizeof(tmp_buf));
                break;

            case MUX_CONFIG_SS_ONLY:
                /* Set mux to USB-only mode state. */
                tmp_buf[1] |= (PI3USB_SS_USB_ONLY_CFG << 1);
                status = Mux_I2C_Write (mux_i2c_address[port], tmp_buf, sizeof(tmp_buf));
                break;

            case MUX_CONFIG_DP_4_LANE:
                /* Set mux to 4-lane DP mode state. */
                tmp_buf[1] |= (PI3USB_DP_4_LANE_CFG << 1);
                status = Mux_I2C_Write (mux_i2c_address[port], tmp_buf, sizeof(tmp_buf));
                break;

            case  MUX_CONFIG_DP_2_LANE:
                /* Set mux to to 2-lane DP plus USB-only mode state. */
                tmp_buf[1] |= (PI3USB_DP_2_LANE_CFG << 1);
                status = Mux_I2C_Write (mux_i2c_address[port], tmp_buf, sizeof(tmp_buf));
                break;

            default:
                break;
        }

        /* Update current mux state. */
        mux_state[port] = MAKE_MUX_STATE(polarity, cfg);
    }

    return (status);
}

/* Initialize the MUX control SCB block. */
bool mux_ctrl_init(uint8_t port)
{
    bool status = true;

    /* Configure the I2C interface. */
    MUX_CTRL_CTRL_REG     = MUX_CTRL_I2C_DEFAULT_CTRL;
    MUX_CTRL_I2C_CTRL_REG = MUX_CTRL_I2C_DEFAULT_I2C_CTRL;

    /* Configure TX direction. */
    MUX_CTRL_TX_CTRL_REG      = MUX_CTRL_I2C_DEFAULT_TX_CTRL;
    MUX_CTRL_TX_FIFO_CTRL_REG = MUX_CTRL_I2C_DEFAULT_TX_FIFO_CTRL;

    MUX_CTRL_CTRL_REG |= MUX_CTRL_CTRL_ENABLED;

    /* Force the MUX into ISOLATE state. */
    status = mux_ctrl_set_cfg (port, MUX_CONFIG_ISOLATE, 0);
    return status;
}

/* End of file */
